﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace RolandBaer.LinkedList
{
    public class LinkedList<T> : IList<T>
    {
        public int IndexOf(T item)
        {
            int pos = 0;
            LinkedListElement< T > temp = first;

            if (EqualityComparer< T >.Default.Equals(temp.Element, item))
                return pos;
            while (temp.Next != null)
            {
                temp = temp.Next;
                pos++;
                if (EqualityComparer< T >.Default.Equals(temp.Element, item))
                    return pos;
            }

            return -1;
        }

        public void Insert(int index, T item)
        {
            if (index < 0 || index > Count) throw new ArgumentOutOfRangeException();

            if (index == Count)
            {
                Add(item);
                return;
            }

            LinkedListElement< T > element = new LinkedListElement< T >();
            element.Element = item;

            if(index == 0)
            {
                element.Next = first;
                first = element;
                return;
            }

            LinkedListElement< T > priorIndex = GetAt(index - 1);
            element.Next = priorIndex.Next;
            priorIndex.Next = element;
        }

        public void RemoveAt(int index)
        {
            if (index < 0 || index >= Count) throw new ArgumentOutOfRangeException();
            if (index == 0)
            {
                first = first.Next;
                return;
            }

            LinkedListElement< T > priorIndex = GetAt(index - 1);
            priorIndex.Next = priorIndex.Next.Next;
        }

        public T this[int index]
        {
            get
            {
                if (index >= Count) throw new ArgumentOutOfRangeException();

                LinkedListElement< T > temp = GetAt(index);

                return temp.Element;
            }
            set
            {
                if (index >= Count) throw new ArgumentOutOfRangeException();

                LinkedListElement< T > temp = GetAt(index);

                temp.Element = value;
            }
        }

        public void Add(T item)
        {
            if (first == null)
            {
                first = new LinkedListElement< T >() { Element = item };
            }
            else
            {
                LinkedListElement< T > temp = first;

                while(temp.Next != null)
                {
                    temp = temp.Next;
                }

                temp.Next = new LinkedListElement< T > { Element = item };
            }
        }

        public void Clear()
        {
            first = null;
        }

        public bool Contains(T item)
        {
            return IndexOf(item) > -1;
        }

        public void CopyTo(T[] array, int arrayIndex)
        {
            if (array == null) throw new ArgumentNullException("array");
            if (arrayIndex < 0) throw new ArgumentOutOfRangeException("arrayIndex");
            if (arrayIndex + Count > array.Length) throw new ArgumentException("The number of elements in the source collection is greater than the available space from arrayIndex to the end of the destination array."); 

            foreach (T item in this)
            {
                array[arrayIndex++] = item;
            }
        }

        public int Count
        {
            get 
            { 
                if (first == null) return 0;

                int count = 1;
                LinkedListElement<T> temp = first;

                while (temp.Next != null)
                {
                    temp = temp.Next;
                    count++;
                }

                return count;
            }
        }

        public bool IsReadOnly
        {
            get { return false; }
        }

        public bool Remove(T item)
        {
            int pos = IndexOf(item);
            if (pos < 0) return false;

            RemoveAt(pos);
            return true;
        }

        public IEnumerator< T > GetEnumerator()
        {
            return new LinkedListEnumerator< T >(this);
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        private LinkedListElement< T > first;

        internal LinkedListElement< T > GetAt(int index)
        {
            LinkedListElement< T > temp = first;

            for (int i = 0; i < index; i++)
            {
                temp = temp.Next;
            }
            return temp;
        }
    }
}
