using System;
using RolandBaer.LinkedList;
using TSharp.Criteria;

namespace RolandBaer.LinkedListTests
{
    testclass LinkedListTest for LinkedList< int >
    {
        LinkedList< int > sut;
        
        testcontext
        {
            test 
            {
                sut = new LinkedList < int >();
                runtest;
            }
        }
        
        test Count get
        {
            runtest int count = sut.Count;

            assert count == 0;
        }
        
        test Count get 
        {
            sut.Add(5);
            
            runtest int count = sut.Count;

            assert count == 1;
        }
        
        test Count get 
        {
            sut.Add(5);
            sut.Add(8);
                        
            runtest int count = sut.Count;

            assert count == 2;
        }
        
        test Count get 
        {
            for(int i = 0; i < 10; i++)
            {
                sut.Add(5 + i);
            }
                 
            runtest int count = sut.Count;

            assert count == 10;
        }
        
        test this[int index] get
        {
            sut.Add(5);

            runtest int value = sut[0];

            assert value == 5;
        }
        
        test this[int index] get
        {
            runtest int value = sut[0];

            assert thrown ArgumentOutOfRangeException;
        }
        
        test this[int index] get
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest int value = sut[2];

            assert thrown ArgumentOutOfRangeException;
        }
        
        test this[int index] get
        {
            int value1, value2;   
            for(int i = 0; i < 10; i++)
            {
                sut.Add(5 + i);
            }
            
            runtest
            {
                value1 = sut[2];
                value2 = sut[9];
            }
            
            assert value1 == 7;
            assert value2 == 14;
        }
        
        test this[int index] set
        {
            int value1, value2;   
            for(int i = 0; i < 10; i++)
            {
                sut.Add(i);
            }
            
            runtest
            {
                sut[2] = 7;
            }
            
            assert sut[1] == 1;
            assert sut[2] == 7;
            assert sut[3] == 3;
        }
        
        test this[int index] set
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest sut[2] = 3;

            assert thrown ArgumentOutOfRangeException;
        }
        
        test Clear()
        {
            sut.Add(5);
            sut.Add(8);
            assert sut.Count == 2;
            
            runtest sut.Clear();

            assert sut.Count == 0; 
        }
        
        test Contains(int)
        {
            sut.Add(5);
            sut.Add(8);

            runtest bool res = sut.Contains(5);
            
            assert res;
        }
        
        test Contains(int)
        {
            sut.Add(5);
            sut.Add(8);

            runtest bool res = sut.Contains(6);
            
            assert res == false;
        }
        
        test Contains(int)
        {
            sut.Add(5);
            sut.Add(8);

            runtest bool res = sut.Contains(8);
            
            assert res;
        }
        
        test IndexOf(int)
        {
            sut.Add(5);
            sut.Add(8);

            runtest int pos = sut.IndexOf(5);
            
            assert pos == 0;
        }
        
        test IndexOf(int)
        {
            sut.Add(5);
            sut.Add(8);

            runtest int pos = sut.IndexOf(6);
            
            assert pos == -1;
        }
        
        test IndexOf(int)
        {
            sut.Add(5);
            sut.Add(8);

            runtest int pos = sut.IndexOf(8);
            
            assert pos == 1;
        }

        test GetEnumerator()
        {
            runtest System.Collections.Generic.IEnumerator<int> enumerator = sut.GetEnumerator();
            
            assert enumerator != null;
        }
                
        test GetEnumerator()
        {
            runtest System.Collections.IEnumerator enumerator = ((System.Collections.IEnumerable)sut).GetEnumerator();
            
            assert enumerator != null;
        }
        
        test CopyTo(int[], int)  
        {
            sut.Add(5);
            sut.Add(8);
            int[] res = new int[2];
            runtest sut.CopyTo(res, 0);

            assert res[0] == 5;
            assert res[1] == 8;
        }
        
        test CopyTo(int[], int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            int[] res = new int[6];
            runtest sut.CopyTo(res, 4);

            assert res[4] == 5;
            assert res[5] == 8;
        }
        
        test CopyTo(int[], int)  
        {
            runtest sut.CopyTo(null, 0);

            assert thrown ArgumentNullException;
        }
        
        test CopyTo(int[], int)  
        {
            int[] res = new int[5];
            runtest sut.CopyTo(res, -1);

            assert thrown ArgumentOutOfRangeException;
        }
                
        test CopyTo(int[], int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            int[] res = new int[5];
            runtest sut.CopyTo(res, 4);

            assert thrown ArgumentException;
        }
        
        test Insert(int, int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest sut.Insert(1, 4);

            assert sut[1] == 4;
            assert sut[2] == 8;
        }   
        
        test Insert(int, int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest sut.Insert(0, 4);

            assert sut[0] == 4;
            assert sut[1] == 5;
        }   
        
        test Insert(int, int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest sut.Insert(2, 4);

            assert sut[1] == 8;
            assert sut[2] == 4;
        }
 
        test Insert(int, int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest sut.Insert(-1, 4);

            assert thrown ArgumentOutOfRangeException;
        }
        
        test Insert(int, int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest sut.Insert(3, 4);

            assert thrown ArgumentOutOfRangeException;
        }
        
        test RemoveAt(int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest sut.RemoveAt(0);

            assert sut.Count == 1;
            assert sut[0] == 8;
        }
             
        test RemoveAt(int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest sut.RemoveAt(1);

            assert sut.Count == 1;
            assert sut[0] == 5;
        }      
          
        test RemoveAt(int)  
        {
            sut.Add(5);
            sut.Add(8);
            sut.Add(4);
            
            runtest sut.RemoveAt(1);

            assert sut[0] == 5;
            assert sut[1] == 4;
        }
        
        test RemoveAt(int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest sut.RemoveAt(2);

            assert thrown ArgumentOutOfRangeException;
        }       
        
        test RemoveAt(int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest sut.RemoveAt(-1);

            assert thrown ArgumentOutOfRangeException;
        }       
          
        test Remove(int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest bool res = sut.Remove(5);

            assert res;
            assert sut.Count == 1;
            assert sut[0] == 8;
        }
          
        test Remove(int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest bool res = sut.Remove(8);

            assert res;
            assert sut.Count == 1;
            assert sut[0] == 5;
        }
        
        test Remove(int)  
        {
            sut.Add(5);
            sut.Add(8);
            
            runtest bool res = sut.Remove(3);

            assert res == false;
            assert sut.Count == 2;
            assert sut[0] == 5;
            assert sut[1] == 8;
        }
        
        test Remove(int)  
        {
            sut.Add(5);
            sut.Add(8);
            sut.Add(5);
                        
            runtest bool res = sut.Remove(5);

            assert res;
            assert sut.Count == 2;
            assert sut[0] == 8;
            assert sut[1] == 5;
        }
        
        test IsReadOnly get
        {
            runtest bool res = sut.IsReadOnly;
            
            assert res == false;
        }
    }
}
